unit ScreenSaveColorBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Spin, ExtCtrls;

type
  TFormSaveColorBox = class(TForm)
    ShapeColorBox: TShape;
    SpinEditnm: TSpinEdit;
    SpinEditPixels: TSpinEdit;
    LabelPixels: TLabel;
    LabelNanometers: TLabel;
    BitBtnOK: TBitBtn;
    BitBtnCancel: TBitBtn;
    LabelWavelength: TLabel;
    LabelFrequency: TLabel;
    SpinEditTHz: TSpinEdit;
    LabelTHz: TLabel;
    LabelRGB: TLabel;
    procedure SpinEditnmChange(Sender: TObject);
    procedure SpinEditTHzChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    UpdatingFrequency :  BOOLEAN;
    UpdatingWavelength:  BOOLEAN;
  public
    { Public declarations }
  end;

var
  FormSaveColorBox: TFormSaveColorBox;

implementation
{$R *.DFM}

  USES
    SpectraLibrary;

procedure TFormSaveColorBox.SpinEditnmChange(Sender: TObject);
  VAR
    R,G,B     :  BYTE;
    wavelength:  INTEGER;
begin
  IF   NOT UpdatingWavelength   // block recursion
  THEN BEGIN
    UpdatingWavelength := TRUE;

    // conversion error can occur if keyboard entry allowed
    TRY
      wavelength := SpinEditnm.Value;
    EXCEPT
      wavelength := 0
    END;

    IF   wavelength < SpinEditnm.MinValue
    THEN BEGIN
      wavelength := SpinEditnm.MinValue;
      SpinEditnm.Value := SpinEditnm.MinValue;
    END;

    IF   wavelength > SpinEditnm.MaxValue
    THEN BEGIN
      wavelength := SpinEditnm.MaxValue;
      SpinEditnm.Value := SpinEditnm.MaxValue;
    END;

    SpinEditTHz.Value := ROUND( WaveLengthToFrequency(wavelength) );

    WavelengthToRGB(wavelength, R,G,B);
    LabelRGB.Caption := RGBToString(R,G,B);
    ShapeColorBox.Brush.Color := RGB(R, G, B);

    UpdatingWavelength := FALSE
  END;
end;

procedure TFormSaveColorBox.SpinEditTHzChange(Sender: TObject);
  VAR
    frequency:  INTEGER;
    R,G,B    :  BYTE;
begin
  IF   NOT UpdatingFrequency   // block recursion
  THEN BEGIN
    UpdatingFrequency := TRUE;

     // conversion error can occur if keyboard entry allowed
    TRY
      frequency := SpinEditTHz.Value;
    EXCEPT
      frequency := 0
    END;

    IF   frequency < SpinEditTHz.MinValue
    THEN BEGIN
      frequency := SpinEditTHz.MinValue;
      SpinEditTHz.Value := SpinEditTHz.MinValue;
    END;

    IF   frequency > SpinEditTHz.MaxValue
    THEN BEGIN
      frequency := SpinEditTHz.MaxValue;
      SpinEditTHz.Value := SpinEditTHz.MaxValue;
    END;

    SpinEditnm.Value := ROUND( FrequencyToWaveLength(frequency) );

    FrequencyToRGB(frequency, R,G,B);
    LabelRGB.Caption := RGBToString(R,G,B);
    ShapeColorBox.Brush.Color := RGB(R, G, B);

    UpdatingFrequency := FALSE
  END
end;


procedure TFormSaveColorBox.FormCreate(Sender: TObject);
begin
  UpdatingWavelength := FALSE;
  UpdatingFrequency  := FALSE
end;

end.
