// Spectra
// Copyright (C) 1998, Earl F. Glynn, Overland Park, KS.
// May be copied freely for non-commercial use.
// E-Mail:  EarlGlynn@att.net

unit ScreenSpectra;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Spin, ComCtrls, Buttons, TeEngine, Series, TeeProcs,
  Chart, SpectraLibrary, ExtDlgs;

type
  TFormSpectra = class(TForm)
    PageControlSpectra: TPageControl;
    TabSheetVisibleLight: TTabSheet;
    TabSheetRGBGraph: TTabSheet;
    ImageSpectra: TImage;
    SpinEditBlue: TSpinEdit;
    LabelWavelength: TLabel;
    SpinEditRed: TSpinEdit;
    TabAtomicSpectra: TTabSheet;
    ChartRGBWavelength: TChart;
    SeriesRed: TLineSeries;
    SeriesGreen: TLineSeries;
    SeriesBlue: TLineSeries;
    LabelSourceCredit1: TLabel;
    ButtonSaveBMP: TButton;
    CheckBoxWavelength: TCheckBox;
    SpinEditWavelength: TSpinEdit;
    LabelIntervalnm: TLabel;
    SeriesIntensity: TLineSeries;
    SeriesY: TLineSeries;
    CheckBoxIntensity: TCheckBox;
    CheckBoxY: TCheckBox;
    ButtonPrintGraph: TButton;
    CheckBoxPhotoPlateOrder: TCheckBox;
    LabelLow: TLabel;
    LabelHigh: TLabel;
    LabelInfo2A: TLabel;
    LabelLambda: TLabel;
    LabelInfo3: TLabel;
    LabelInfo4A: TLabel;
    LabelInfo4B: TLabel;
    LabelHiddenComment1: TLabel;
    LabelHiddenComment2: TLabel;
    LabelHiddenComment3: TLabel;
    LabelInfo5: TLabel;
    LabelInfo6: TLabel;
    LabelInfo7: TLabel;
    LabelInfo8: TLabel;
    LabelInfo9: TLabel;
    LabelInfo10: TLabel;
    LabelInfo1: TLabel;
    LabelInfo2B: TLabel;
    LabelInfoSource1: TLabel;
    LabelInfoSource2: TLabel;
    LabelInfo11: TLabel;
    LabelInfo12: TLabel;
    LabelInfo13: TLabel;
    LabelInfo14: TLabel;
    RadioGroupSpectra: TRadioGroup;
    SavePictureDialogSpectrum: TSavePictureDialog;
    ButtonPrintSpectra: TButton;
    Label256ColorComment: TLabel;
    LabelRGB: TLabel;
    LabelLab1: TLabel;
    LabelLab2: TLabel;
    LabelSourceCredit2: TLabel;
    ButtonColorBox: TButton;
    ShapeColorBox: TShape;
    LabelColorBoxNm: TLabel;
    Shape1: TShape;
    TabSheetDisclaimer: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    LabelFrequency: TLabel;
    CheckBoxFrequency: TCheckBox;
    LabelIntervalTHz: TLabel;
    SpinEditFrequency: TSpinEdit;
    LabelColorBoxTHz: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpinEditWavelengthChange(Sender: TObject);
    procedure SpinEditIntervalChange(Sender: TObject);
    procedure CheckBoxClick(Sender: TObject);
    procedure CheckBoxIntensityClick(Sender: TObject);
    procedure CheckBoxYClick(Sender: TObject);
    procedure ButtonPrintGraphClick(Sender: TObject);
    procedure RadioGroupSpectraClick(Sender: TObject);
    procedure ButtonSaveBMPClick(Sender: TObject);
    procedure ButtonPrintSpectraClick(Sender: TObject);
    procedure ImageSpectraMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure LabelLab2Click(Sender: TObject);
    procedure LabelSourceCredit2Click(Sender: TObject);
    procedure ButtonColorBoxClick(Sender: TObject);
    procedure ShapeColorBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CheckBoxFrequencyClick(Sender: TObject);
  private
    { Private declarations }
    BitmapSpectrum    :  TBitmap;
    ColorBoxnm        :  INTEGER;
    ColorBoxRGB       :  STRING;
    VisibleBalmerLines:  ARRAY[3..9] OF NanoMeters;
    NormalLeftBlue    :  INTEGER;
    NormalLeftRed     :  INTEGER;
    PROCEDURE UpdateImage;
  public
    { Public declarations }
  end;

var
  FormSpectra: TFormSpectra;

implementation
{$R *.DFM}

  USES
    Printers,
    ShellAPI,              // ShellExecute
    ScreenSaveColorBox,
    ScreenWavelength;


  // Based on posting to borland.public.delphi.winapi by Rodney E Geraghty, 8/8/97.
  // Used to print bitmap on any Windows printer.
  PROCEDURE PrintBitmap(Canvas:  TCanvas; DestRect:  TRect;  Bitmap:  TBitmap);
    VAR
      BitmapHeader:  pBitmapInfo;
      BitmapImage :  POINTER;
      HeaderSize  :  DWORD;       // Use DWORD for D3/D4/D5 compatibility
      ImageSize   :  DWORD;
  BEGIN
    GetDIBSizes(Bitmap.Handle, HeaderSize, ImageSize);
    GetMem(BitmapHeader, HeaderSize);
    GetMem(BitmapImage,  ImageSize);
    TRY
      GetDIB(Bitmap.Handle, Bitmap.Palette, BitmapHeader^, BitmapImage^);
      StretchDIBits(Canvas.Handle,
                    DestRect.Left, DestRect.Top,     {Destination Origin}
                    DestRect.Right  - DestRect.Left, {Destination Width}
                    DestRect.Bottom - DestRect.Top,  {Destination Height}
                    0, 0,                            {Source Origin}
                    Bitmap.Width, Bitmap.Height,     {Source Width & Height}
                    BitmapImage,
                    TBitmapInfo(BitmapHeader^),
                    DIB_RGB_COLORS,
                    SRCCOPY)
    FINALLY
      FreeMem(BitmapHeader);
      FreeMem(BitmapImage)
    END
  END {DrawBitmap};


  FUNCTION  GetPixelDepth:  INTEGER;
    VAR
      DeviceContext:  hDC;
  BEGIN
    // Get the screen's DC since memory DCs are not reliable
    DeviceContext := GetDC(hWnd_DeskTop);

    TRY
      RESULT := GetDeviceCaps(DeviceContext, BITSPIXEL)
    FINALLY
      // Give back the screen DC
      ReleaseDC(0, DeviceContext)
    END;
  END {GetPixelDepth};


procedure TFormSpectra.FormCreate(Sender: TObject);
  VAR
    n         :  INTEGER;
    R,G,B     :  BYTE;
    Wavelength:  INTEGER;
begin
  NormalLeftBlue := SpinEditBlue.Left;
  NormalLeftRed  := SpinEditRed.Left;

  LabelRGB.Caption := '';
  PageControlSpectra.ActivePage := TabSheetVisibleLight;

  ColorBoxnm := 580;
  ColorBoxRGB := '';

  Label256ColorComment.Visible := (GetPixelDepth < 16);

  FOR n := Low(VisibleBalmerLines) TO High(VisibleBalmerLines) DO
    VisibleBalmerLines[n] := 364.7054 * SQR(n) / (SQR(n) - 4);

  BitmapSpectrum := TBitmap.Create;
  BitmapSpectrum.Width  := ImageSpectra.Width;
  BitmapSpectrum.Height := ImageSpectra.Height;
  BitmapSPectrum.PixelFormat := pf24bit;
  UpdateImage;

  FOR Wavelength := WavelengthMinimum TO WavelengthMaximum DO
  BEGIN
    WavelengthToRGB(Wavelength, R,G,B);
    SeriesRed.AddXY(Wavelength, R, '', clTeeColor);
    SeriesGreen.AddXY(Wavelength, G, '', clTeeColor);
    SeriesBlue.AddXY(Wavelength, B, '', clTeeColor);
    SeriesIntensity.AddXY(Wavelength, ROUND( (R+G+B)/3 ), '', clTeeColor);
    SeriesY.AddXY(Wavelength, ROUND(0.299*R + 0.587*G + 0.114*B), '', clTeeColor)
  END
end;


procedure TFormSpectra.FormDestroy(Sender: TObject);
begin
  BitmapSpectrum.Free
end;


PROCEDURE TFormSpectra.UpdateImage;

  TYPE
    TRGBTripleArray = ARRAY[WORD] OF TRGBTriple;
    pRGBTripleArray = ^TRGBTripleArray;

  VAR
    B             :  BYTE;
    BlueWavelength:  Nanometers;
    G             :  BYTE;
    flags         :  pByteArray;
    Frequency     :  TeraHertz;
    FrequencyBlue :  INTEGER;
    FrequencyRed  :  INTEGER;
    i             :  INTEGER;
    index         :  INTEGER;
    j             :  INTEGER;
    R             :  BYTE;
    RedWavelength :  Nanometers;
    Row           :  pRGBTripleArray;
    s             :  STRING;
    Wavelength    :  Nanometers;
BEGIN
  BlueWavelength := SpinEditBlue.Value;
  RedWavelength  := SpinEditRed.Value;

  // Set flags if hydrogen emission or absorption spectra
  IF   RadioGroupSpectra.ItemIndex = 0
  THEN flags := NIL   // Do this to avoid compiler warning
  ELSE BEGIN
    // Dynamically allocate one flag for each column in bitmap
    GetMem(flags, ImageSpectra.Width);
    FOR i := 0 TO ImageSpectra.Width-1 DO
      flags[i] := 0;

    FOR j := Low(VisibleBalmerLines) TO High(VisibleBalmerLines) DO
    BEGIN
      index := ROUND((ImageSpectra.Width-1) *
                     (VisibleBalmerLines[j] - SpinEditBlue.Value) /
                     (SpinEditRed.Value     - SpinEditBlue.Value));
      IF   (index >= 0) AND (index < ImageSpectra.Width)
      THEN flags[index] := 1
    END

  END;

  FOR i := 0 TO BitmapSpectrum.Width-1 DO
  BEGIN
    IF  CheckBoxPhotoPlateOrder.Checked
    THEN index := BitmapSpectrum.Width - 1 - i
    ELSE index := i;

    Wavelength := BlueWavelength +
                  (RedWavelength-BlueWavelength)* i / BitmapSpectrum.Width;

    WavelengthToRGB(Wavelength, R,G,B);

    // Fix "line" for Emission Spectra
    IF   (RadioGroupSpectra.ItemIndex = 1) AND (Flags[i] = 0)
    THEN BEGIN
      R := 0;
      G := 0;
      B := 0
    END;

    // Fix "line" for Absorption Spectra
    IF   (RadioGroupSpectra.ItemIndex = 2) AND (Flags[i] = 1)
    THEN BEGIN
      R := 0;
      G := 0;
      B := 0
    END;

    FOR j := 0 TO BitmapSpectrum.Height-1 DO
    BEGIN
      row := BitmapSpectrum.Scanline[j];
      WITH row[index] DO
      BEGIN
        rgbtRed   := R;
        rgbtGreen := G;
        rgbtBlue  := B
      END
    END

  END;

  // Wavelength Tick Marks
  IF   CheckBoxWavelength.Checked  AND (SpinEditWavelength.Value > 0)
  THEN BEGIN
    Wavelength := (SpinEditBlue.Value DIV SpinEditWavelength.Value) *
                         SpinEditWavelength.Value;


    BitmapSpectrum.Canvas.Font.Size  := 8;
    BitmapSpectrum.Canvas.Brush.Style := bsClear;

    WHILE (Wavelength <= SpinEditRed.Value) DO
    BEGIN
      // Empirical adjustment to make sure text contrast is always good
      IF   (Wavelength >= 470) AND (Wavelength <=620) AND
           ((RadioGroupSpectra.ItemIndex = 0) OR (RadioGroupSpectra.ItemIndex = 2))
      THEN BEGIN
        // Black on very light greens and yellow provides good contrast
        BitmapSpectrum.Canvas.Pen.Color := clBlack;
        BitmapSpectrum.Canvas.Font.Color := clBlack
      END
      ELSE BEGIN
        // White on darker reds and blues provides good contrast
        BitmapSpectrum.Canvas.Pen.Color := clWhite;
        BitmapSpectrum.Canvas.Font.Color := clWhite
      END;

      i := ROUND(BitmapSpectrum.Width * (Wavelength        - SpinEditBlue.Value) /
                                (SpinEditRed.Value - SpinEditBlue.Value));

      IF  CheckBoxPhotoPlateOrder.Checked
      THEN index := BitmapSpectrum.Width -1 - i
      ELSE index := i;


      BitmapSpectrum.Canvas.MoveTo(index, BitmapSpectrum.Height-1);
      j := 8*BitmapSpectrum.Height DIV 10;
      BitmapSpectrum.Canvas.LineTo(index, j);
      s := IntToStr(TRUNC(Wavelength));
      BitmapSpectrum.Canvas.TextOut(index - BitmapSpectrum.Canvas.TextWidth(s) DIV 2,
                            j - BitmapSpectrum.Canvas.TextHeight(s),  s);

      Wavelength := Wavelength + SpinEditWavelength.Value
    END

  END;

  // Frequency Tick Marks
  IF   CheckBoxFrequency.Checked  AND (SpinEditFrequency.Value > 0)
  THEN BEGIN
    FrequencyBlue := ROUND( WavelengthToFrequency(SpinEditBlue.Value) );
    FrequencyRed  := ROUND( WavelengthToFrequency(SpinEditRed.Value)  );

    Frequency :=     (FrequencyRed DIV SpinEditFrequency.Value) *
                     SpinEditFrequency.Value;

    Wavelength := FrequencyToWavelength(Frequency);

    BitmapSpectrum.Canvas.Font.Size  := 8;
    BitmapSpectrum.Canvas.Brush.Style := bsClear;

    WHILE (Frequency <= FrequencyBlue) DO
    BEGIN
      // Empirical adjustment to make sure text contrast is always good
      IF   (Wavelength >= 470) AND (Wavelength <=620) AND
           ((RadioGroupSpectra.ItemIndex = 0) OR (RadioGroupSpectra.ItemIndex = 2))
      THEN BEGIN
        // Black on very light greens and yellow provides good contrast
        BitmapSpectrum.Canvas.Pen.Color := clBlack;
        BitmapSpectrum.Canvas.Font.Color := clBlack
      END
      ELSE BEGIN
        // White on darker reds and blues provides good contrast
        BitmapSpectrum.Canvas.Pen.Color := clWhite;
        BitmapSpectrum.Canvas.Font.Color := clWhite
      END;

      i := ROUND(BitmapSpectrum.Width * (Wavelength        - SpinEditBlue.Value) /
                                (SpinEditRed.Value - SpinEditBlue.Value));

      IF  CheckBoxPhotoPlateOrder.Checked
      THEN index := BitmapSpectrum.Width -1 - i
      ELSE index := i;

      BitmapSpectrum.Canvas.MoveTo(index, 0);
      j := 2*BitmapSpectrum.Height DIV 10;
      BitmapSpectrum.Canvas.LineTo(index, j);
      s := IntToStr(TRUNC(Frequency));
      BitmapSpectrum.Canvas.TextOut(index - BitmapSpectrum.Canvas.TextWidth(s) DIV 2,
                            j,  s);

      Frequency := Frequency + SpinEditFrequency.Value;
      Wavelength := FrequencyToWavelength(Frequency)
    END

  END;

  ImageSpectra.Picture.Graphic := BitmapSpectrum;

  // Free flags if hydrogen emission or absorption spectra
  IF   RadioGroupSpectra.ItemIndex > 0
  THEN BEGIN
    FreeMem(Flags)
  END
END {UpdateImage};


procedure TFormSpectra.SpinEditWavelengthChange(Sender: TObject);
  VAR
    Increment:  INTEGER;
begin
  // Make sure Red > Blue always
  IF   SpinEditRed.Value - SpinEditBlue.Value < 10
  THEN
    IF   (Sender AS TSpinEdit) = SpinEditBlue
    THEN SpinEditRed.Value  := SpinEditBlue.Value + 10
    ELSE SpinEditBlue.Value := SpinEditRed.Value  - 10;

  // Frequency Delta
  CASE SpinEditRed.Value - SpinEditBlue.Value OF
      0..  2:  Increment :=   0;
      3.. 10:  Increment :=   2;
     11.. 20:  Increment :=   5;
     21..100:  Increment :=  10;
    101..800:  Increment :=  50;
    ELSE       Increment :=   0    // Negative Range
  END;

  SpinEditFrequency.Value := Increment;

  // Wavelength Delta
  CASE SpinEditRed.Value - SpinEditBlue.Value OF
      0..  2:  Increment :=   0;
      3.. 10:  Increment :=   1;
     11.. 20:  Increment :=   5;
     21..100:  Increment :=  10;
    101..800:  Increment :=  50;
    ELSE       Increment :=   0    // Negative Range
  END;

  SpinEditWavelength.Value := Increment;
  SpinEditWavelength.MinValue :=   // Multiple of 5, about 4% of current width
     (MulDiv( ABS(SpinEditRed.Value - SpinEditBlue.Value), 4, 100) DIV 5) * 5;

  CheckBoxWavelength.Visible := (SpinEditWavelength.Value > 0);
  SpinEditWavelength.Visible := CheckBoxWavelength.Visible;
  LabelIntervalnm.Visible    := CheckBoxWavelength.Visible;

  IF   NOT ShapeColorBox.Visible
  THEN ColorBoxnm := (SpinEditRed.Value + SpinEditBlue.Value) DIV 2;

  UpdateImage;
end;


procedure TFormSpectra.SpinEditIntervalChange(Sender: TObject);
begin
  UpdateImage
end;

procedure TFormSpectra.CheckBoxClick(Sender: TObject);
begin
  UpdateImage;

  // Clumsy, but switch the location of these
  IF   CheckBoxPhotoPlateOrder.Checked
  THEN BEGIN
    SpinEditBlue.Left := NormalLeftRed;
    SpinEditRed.Left  := NormalLeftBlue;

    LabelLow.Left     := NormalLeftRed  + 5;
    LabelHigh.Left    := NormalLeftBlue + 5;
  END
  ELSE BEGIN
    SpinEditBlue.Left := NormalLeftBlue;
    SpinEditRed.Left  := NormalLeftRed;

    LabelLow.Left     := NormalLeftBlue + 5;
    LabelHigh.Left    := NormalLeftRed  + 5;
  END
end;

procedure TFormSpectra.CheckBoxIntensityClick(Sender: TObject);
begin
  SeriesIntensity.Active := CheckBoxIntensity.Checked
end;

procedure TFormSpectra.CheckBoxYClick(Sender: TObject);
begin
  SeriesY.Active := CheckBoxY.Checked
end;

procedure TFormSpectra.ButtonPrintGraphClick(Sender: TObject);
begin
  ChartRGBWavelength.PrintLandscape
end;


procedure TFormSpectra.RadioGroupSpectraClick(Sender: TObject);
begin
  UpdateImage
end;

procedure TFormSpectra.ButtonSaveBMPClick(Sender: TObject);
begin
  SavePictureDialogSpectrum.Filename := 'Spectrum.BMP';
  IF  SavePictureDialogSpectrum.Execute
  THEN BEGIN
    BitmapSpectrum.SaveToFile(SavePictureDialogSpectrum.Filename);
    ShowMessage('File ' +
                SavePictureDialogSpectrum.Filename +
                ' saved.')
  END
end;

procedure TFormSpectra.ButtonPrintSpectraClick(Sender: TObject);
  VAR
    ImageBottom    :  INTEGER;
    ImageLeft      :  INTEGER;
    ImageRight     :  INTEGER;
    ImageTop       :  INTEGER;
    iPixelsPerInch :  INTEGER;
    jFromTop       :  INTEGER;
    jPixelsPerInch :  INTEGER;
    TargetRectangle:  TRect;
    TextLine       :  STRING;
begin
  Screen.Cursor := crHourGlass;
  TRY
    iPixelsPerInch := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
    jPixelsPerInch := GetDeviceCaps(Printer.Handle, LOGPIXELSY);

    Printer.Orientation := poPortrait;
    Printer.BeginDoc;

      // Let's center a 6" spectrum 1.5" from top of page
      jFromTop := ROUND(1.50*jPixelsPerInch);

      // Print Title
      Printer.Canvas.Font.Name := 'Arial';
      Printer.Canvas.Font.Size := 14;
      TextLine := RadioGroupSpectra.Items[RadioGroupSpectra.ItemIndex] + ' Spectrum';
      Printer.Canvas.TextOut((Printer.PageWidth - Printer.Canvas.TextWidth(TextLine)) DIV 2,
                            jFromTop,
                            TextLine);
      jFromTop := jFromTop + Printer.Canvas.TextHeight('X');

      // Label numeric values
      IF   CheckBoxFrequency.Checked
      THEN BEGIN
        Printer.Canvas.Font.Size := 10;
        jFromTop := jFromTop + Printer.Canvas.TextHeight('X');
        // Use text shown on screen
        TextLine := LabelFrequency.Caption;
        Printer.Canvas.TextOut((Printer.PageWidth - Printer.Canvas.TextWidth(TextLine)) DIV 2,
                               jFromTop,
                               TextLine);
        jFromTop := jFromTop + 3*Printer.Canvas.TextHeight('X') DIV 2;
      END;


      ImageLeft := Printer.PageWidth DIV 2 -       // Center of page
                   3*iPixelsPerInch;               // 3" left
      ImageRight := ImageLeft + 6*iPixelsPerInch;  // 6" long spectrum
      ImageTop := jFromTop;
      ImageBottom := ImageTop +
        ROUND((ImageRight - ImageLeft) * BitmapSpectrum.Height /
                                         BitmapSpectrum.Width);
      TargetRectangle := Rect(ImageLeft, ImageTop, ImageRight, ImageBottom);

      // Print the Spectrum Image
      PrintBitmap(Printer.Canvas, TargetRectangle, BitmapSpectrum);

      // Label numeric values
      IF   CheckBoxWavelength.Checked
      THEN BEGIN
        Printer.Canvas.Font.Size := 10;
        // Use text shown on screen
        TextLine := LabelWavelength.Caption;
        Printer.Canvas.TextOut((Printer.PageWidth - Printer.Canvas.TextWidth(TextLine)) DIV 2,
                               ImageBottom + Printer.Canvas.TextHeight('X') DIV 2,
                               TextLine)
      END;

      // Print Footer
      Printer.Canvas.Font.Size := 8;
      TextLine := FormatDateTime('m/d/yyyy h:nn', Now);
      Printer.Canvas.TextOut(ImageLeft, Printer.PageHeight-
                                        Printer.Canvas.TextHeight('X'),
                             TextLine);
    Printer.EndDoc
  FINALLY
    Screen.Cursor := crDefault
  END;

  ShowMessage('Spectrum printed')
end;


procedure TFormSpectra.ImageSpectraMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
  VAR
    color:  TColor;
    R,G,B:  BYTE;
begin
  LabelColorBoxNm.Visible := TRUE;
  LabelColorBoxTHz.Visible := TRUE;

  IF   CheckBoxPhotoPlateOrder.Checked
  THEN ColorBoxnm := Round(SpinEditRed.Value  + x*(SpinEditBlue.Value - SpinEditRed.Value)/(ImageSpectra.Width-1))
  ELSE ColorBoxnm := Round(SpinEditBlue.Value + x*(SpinEditRed.Value - SpinEditBlue.Value)/(ImageSpectra.Width-1));
  LabelColorBoxNm.Caption  := IntToStr(ColorBoxnm) + ' nm';
  LabelColorBoxTHz.Caption := IntToStr( ROUND( WavelengthToFrequency(ColorBoxnm) ) ) + ' THz';

  WavelengthToRGB(ColorBoxnm, R,G,B);
  color := RGB(R,G,B);

  ColorBoxRGB := RGBToString(R,G,B);
  LabelRGB.Caption := ColorBoxRGB;                  

  ShapeColorBox.Visible := TRUE;
  ButtonColorBox.Enabled := TRUE;                    
  ShapeColorBox.Brush.Color := color;
end;

procedure TFormSpectra.LabelLab2Click(Sender: TObject);
begin
 ShellExecute(0, 'open', pchar('http://www.efg2.com/lab'),
               NIL, NIL, SW_NORMAL)
end;

procedure TFormSpectra.LabelSourceCredit2Click(Sender: TObject);
begin
  ShellExecute(0, 'open', pchar('http://www.physics.sfasu.edu/astro/color.html'),
               NIL, NIL, SW_NORMAL)
end;

procedure TFormSpectra.ButtonColorBoxClick(Sender: TObject);
  VAR
    Bitmap:  TBitmap;
    R,G,B :  BYTE;
begin
  FormSaveColorBox.SpinEditnm.Value := ColorBoxnm;
  FormSaveColorBox.ShapeColorBox.Brush.Color := ShapeColorBox.Brush.Color;
  FormSaveColorBox.ShowModal;
  IF   FormSaveColorBox.ModalResult = mrOK
  THEN BEGIN

    SavePictureDialogSpectrum.Filename :=
         IntToStr(FormSaveColorBox.SpinEditnm.Value) + 'nm' +
         IntToStr(FormSaveColorBox.SpinEditTHz.Value) + 'THz' +
         '.BMP';

    IF  SavePictureDialogSpectrum.Execute
    THEN BEGIN
      Bitmap := TBitmap.Create;
      TRY
        Bitmap.Width  := FormSaveColorBox.SpinEditPixels.Value;
        Bitmap.Height := Bitmap.Width;
        Bitmap.PixelFormat := pf24bit;

        WavelengthToRGB(FormSaveColorBox.SpinEditnm.Value, R,G,B);
        Bitmap.Canvas.Brush.Color := RGB(R, G, B);
        Bitmap.Canvas.FillRect(Bitmap.Canvas.ClipRect);

        Bitmap.SaveToFile(SavePictureDialogSpectrum.Filename);
        ShowMessage('File ' +
                    SavePictureDialogSpectrum.Filename +
                    ' saved.')
      FINALLY
        Bitmap.Free
      END
    END

  END
end;

procedure TFormSpectra.ShapeColorBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

  VAR
    FormWavelength:  TFormWavelength;
begin
  FormWavelength := TFormWavelength.Create(NIL);
  TRY
    FormWavelength.Caption := 'Wavelength = ' + LabelColorBoxNm.Caption +
                              '          ' +
                              'Frequency = ' + LabelColorBoxTHz.Caption +
                              '          ' +
                              ColorBoxRGB;
    FormWavelength.Color := ShapeColorBox.Brush.Color;
    FormWavelength.ShowModal
  FINALLY
    FormWavelength.Free
  END
end;

procedure TFormSpectra.CheckBoxFrequencyClick(Sender: TObject);
begin
  LabelFrequency.Visible := CheckBoxFrequency.Checked;
  UpdateImage
end;

end.
